-- Survival MOD Script v1
-- Created by Brock Samson
-- Based on Survival Extreme V3FA script

-- import
--------------------------------------------------------------------------
local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua');
local ScenarioFramework = import('/lua/ScenarioFramework.lua');
local Utilities = import('/lua/utilities.lua');


-- class variables
--------------------------------------------------------------------------
local Survival_TickInterval = 0.50; -- how much delay between each script iteration

local Survival_NextSpawnTime = 0;
local Survival_CurrentTime = 0;

local Survival_GameState = 0; -- 0 pre-spawn, 1 post-spawn, 2 player win, 3 player defeat
local Survival_PlayerCount = 0; -- how many human players there are

local Survival_MarkerCounts = {0, 0, 0}; -- 1 center / 2 waypoint / 3 spawn

local Survival_UnitCountPerMinute = 0; -- how many units to spawn per minute (taking into consideration player count)
local Survival_UnitCountPerWave = 0; -- how many units to spawn with each wave (taking into consideration player count)

local Survival_MinWarnTime = 0;

local Survival_HealthBuffLand = 1.00;
local Survival_HealthBuffAir = 1.00;
local Survival_HealthBuffSea = 1.00;
local Survival_HealthBuffGate = 1.00;
local Survival_HealthBuffDefObject = 1.00;

local Survival_DefUnit = nil;
local Survival_DefCheckHP = 0.0;
local Survival_DefLastHP = 0;

-- unit tables {'UnitID', OrderType};
--------------------------------------------------------------------------

-- order types

	-- 1 = move
	-- 2 = attack move
	-- 3 = patrol paths

-- wave table entries are in the following format

-- {"Description", OrderType, 'UnitID'},

-- entry 1 is a description text not used by code
-- entry 2 is the order given to this unit
-- entry 3 is the blueprint id and can be added multiple times as needed
-- when a unit table is randomly selected for spawn ONE unit from within will be chosen at random
-- for example if the "T1 Tank" line is selected ONE of the four tanks will be selected for spawning

-- below are default unit categories but custom ones can be made using same formatting

--	{"T1 Scout", 1, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
--	{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},
--	{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
--	{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
--	{"T1 AA", 3, 'UAL0104', 'URL0104', 'UEL0104', 'XSL0104'},

--	{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'}, -- aeon blaze, cybran rhino, uef pillar, uef mongoose, sera tank
--	{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'}, -- aeon obsidian, cybran wagner, uef riptide, sera bot
--	{"T2 RocketBot", 2, 'DRL0204'},
--	{"T2 AA", 2, 'UAL0205', 'URL0205', 'UEL0205', 'XSL0205'},
--	{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},
--	{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
--	{"T2 Stealth", 3, 'URL0306'},
--	{"T2 Bomb", 2, 'XRL0302'},

--	{"T2 Destroyer", 2, 'URS0201'}, -- cybran destroyer

--	{"T3 Bot1", 4, 'URL0303', 'UEL0303'}, -- cybran loyalist, uef titan
--	{"T3 Bot2", 4, 'UAL0303', 'XSL0303'}, -- aeon harb, sera tank
--	{"T3 Bot3", 4, 'XRL0305', 'XEL0303'}, -- cybran brick, uef percival
--	{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},
--	{"T3 Arty", 2, 'UAL0304', 'URL0304', 'UEL0304', 'XSL0304'},
--	{"T3 Shield", 3, 'XSL0307'},
--	{"T3 MML", 2, 'XEL0306'},
--	{"T3 ShieldKill", 2, 'DAL0310'},

--	{"T3 Subcom", 2, 'UAL0301', 'URL0301', 'UEL0301', 'XSL0301'},

local Survival_WaveTables = {
	{ -- ground
		2; -- current wave id (STARTS AT 2)
		{ -- Wave Set 1

			0.0; -- spawn time

			{"T1 Scout", 3, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},
		},
		{ -- Wave Set 2

			1.0; -- spawn time

			{"T1 Scout", 3, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},
		},
		{ -- Wave Set 3

			2.0; -- spawn time

			{"T1 Scout", 3, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},

			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
		},
		{ -- Wave Set 4

			3.0; -- spawn time

			{"T1 Scout", 3, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},

			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},

			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
		},
		{ -- Wave Set 5

			3.75; -- spawn time

			{"T1 Scout", 3, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
			{"T1 Scout", 3, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},

			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},

			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
		},
		{ -- Wave Set 6

			4.5; -- spawn time

			{"T1 Scout", 3, 'UAL0101', 'URL0101', 'UEL0101', 'XSL0101'},
			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},

			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 1, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'}, -- order change to 1

			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
		},
		{ -- Wave Set 7

			5.25; -- spawn time

			{"T1 Bot", 1, 'UAL0106', 'URL0106', 'UEL0106'},

			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 1, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'}, -- order change to 1

			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},

			{"T2 Bomb", 2, 'XRL0302'},
		},
		{ -- Wave Set 8

			6.0; -- spawn time

			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 4, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'},
			{"T1 Tank", 1, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'}, -- order change to 1

			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},

			{"T2 Bomb", 2, 'XRL0302'},

			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},
		},
		{ -- Wave Set 9

			7.0; -- spawn time

			{"T1 Tank", 1, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'}, -- order change to 1
			{"T1 Tank", 1, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'}, -- order change to 1

			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},
			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},

			{"T2 Bomb", 2, 'XRL0302'},

			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},
			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},

			{"T2 RocketBot", 2, 'DRL0204'},
		},
		{ -- Wave Set 10

			8.0; -- spawn time

			{"T1 Tank", 1, 'UAL0201', 'URL0107', 'UEL0201', 'XSL0201'}, -- order change to 1
			{"T1 Arty", 2, 'UAL0103', 'URL0103', 'UEL0103', 'XSL0103'},

			{"T2 Bomb", 2, 'XRL0302'},

			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},
			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},
			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},
		},
		{ -- Wave Set 11

			9.0; -- spawn time

			{"T2 Bomb", 2, 'XRL0302'},

			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},
			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},
			{"T2 Tank", 1, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'}, -- order change to 1

			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},
		},
		{ -- Wave Set 12

			10.0; -- spawn time

			{"T2 Bomb", 2, 'XRL0302'},

			{"T2 Tank", 4, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'},
			{"T2 Tank", 1, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'}, -- order change to 1
			{"T2 Tank", 1, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'}, -- order change to 1

			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},
		},
		{ -- Wave Set 13

			11.0; -- spawn time

			{"T2 Bomb", 2, 'XRL0302'},

			{"T2 Tank", 1, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'}, -- order change to 1
			{"T2 Tank", 1, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'}, -- order change to 1

			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 1, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'}, -- order change to 1

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
		},
		{ -- Wave Set 14

			12.0; -- spawn time

			{"T2 Tank", 1, 'XAL0203', 'URL0202', 'UEL0202', 'DEL0204', 'XSL0203'}, -- order change to 1

			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 1, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'}, -- order change to 1

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
		},
		{ -- Wave Set 15

			13.0; -- spawn time

			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 1, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'}, -- order change to 1

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
		},
		{ -- Wave Set 16

			14.0; -- spawn time

			{"T2 HeavyTank", 4, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'},
			{"T2 HeavyTank", 1, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'}, -- order change to 1

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},

		},
		{ -- Wave Set 17

			15.0; -- spawn time

			{"T2 HeavyTank", 1, 'UAL0202', 'URL0203', 'UEL0203', 'XSL0202'}, -- order change to 1

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
		},
		{ -- Wave Set 18

			16.0; -- spawn time

			{"T2 RocketBot", 2, 'DRL0204'},
			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1
		},
		{ -- Wave Set 19

			17.0; -- spawn time

			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
		},
		{ -- Wave Set 20

			18.0; -- spawn time

			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
			{"T3 Shield", 3, 'XSL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
			{"T3 Bot3", 1, 'XRL0305', 'XEL0305'}, -- order change to 1
		},
		{ -- Wave Set 21

			19.0; -- spawn time

			{"T2 MML", 2, 'UAL0111', 'URL0111', 'UEL0111', 'XSL0111'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
			{"T3 Shield", 3, 'XSL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
			{"T3 Bot3", 1, 'XRL0305', 'XEL0305'}, -- order change to 1

			{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},
		},
		{ -- Wave Set 22

			20.0; -- spawn time

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
			{"T3 Shield", 3, 'XSL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
			{"T3 Bot3", 1, 'XRL0305', 'XEL0305'}, -- order change to 1

			{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},

			{"T3 MML", 2, 'XEL0306'},
		},
		{ -- Wave Set 23

			24.0; -- spawn time

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
			{"T3 Shield", 3, 'XSL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
			{"T3 Bot3", 1, 'XRL0305', 'XEL0305'}, -- order change to 1

			{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},

			{"T3 MML", 2, 'XEL0306'},

			{"T3 Arty", 2, 'UAL0304', 'URL0304', 'UEL0304', 'XSL0304'},
		},
		{ -- Wave Set 24

			28.0; -- spawn time

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
			{"T3 Shield", 3, 'XSL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
			{"T3 Bot3", 1, 'XRL0305', 'XEL0305'}, -- order change to 1

			{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},

			{"T3 MML", 2, 'XEL0306'},

			{"T3 Arty", 2, 'UAL0304', 'URL0304', 'UEL0304', 'XSL0304'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
			{"T3 Shield", 3, 'XSL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
			{"T3 Bot3", 1, 'XRL0305', 'XEL0305'}, -- order change to 1

			{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},

			{"T3 MML", 2, 'XEL0306'},

			{"T3 Arty", 2, 'UAL0304', 'URL0304', 'UEL0304', 'XSL0304'},

			{"T2 Shield", 3, 'UAL0307', 'UEL0307'},
			{"T3 Shield", 3, 'XSL0307'},

			{"T3 Bot1", 4, 'URL0303', 'UEL0303'},
			{"T3 Bot1", 1, 'URL0303', 'UEL0303'}, -- order change to 1

			{"T3 Bot2", 4, 'UAL0303', 'XSL0303'},
			{"T3 Bot2", 1, 'UAL0303', 'XSL0303'}, -- order change to 1

			{"T3 Bot3", 4, 'XRL0305', 'XEL0305'},
			{"T3 Bot3", 1, 'XRL0305', 'XEL0305'}, -- order change to 1

			{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},

			{"T3 MML", 2, 'XEL0306'},

			{"T3 Arty", 2, 'UAL0304', 'URL0304', 'UEL0304', 'XSL0304'},

			{"Monkeylord", 2, 'URL0402'},
		},
		{ -- Wave Set 23

			32.0; -- spawn time

			{"Monkeylord", 2, 'URL0402'},
			{"Monkeylord", 1, 'URL0402'},
		},
	},
};
--	{"T3 Bot1", 4, 'URL0303', 'UEL0303'}, -- cybran loyalist, uef titan
--	{"T3 Bot2", 4, 'UAL0303', 'XSL0303'}, -- aeon harb, sera tank
--	{"T3 Bot3", 4, 'XRL0305', 'XEL0305'}, -- cybran brick, uef percival
--	{"T3 Sniper", 2, 'XAL0305', 'XSL0305'},
--	{"T3 Arty", 2, 'UAL0304', 'URL0304', 'UEL0304', 'XSL0304'},
--	{"T3 Shield", 3, 'XSL0307'},
--	{"T3 MML", 2, 'XEL0306'},
--	{"T3 ShieldKill", 2, 'DAL0310'},
-- 	{"Monkeylord", 2, 'URL0402'},


-- called at start to read various settings
--------------------------------------------------------------------------
function OnPopulate()

	LOG("----- Survival MOD: OnPopulate()");

	-- start the armies
	ScenarioUtils.InitializeArmies();

	-- prepare all the survival stuff
	Survival_InitGame();

	-- ScenarioFramework.SetPlayableArea('AREA_1' , false) -- restrict playable area because the map is too big

end



-- econ adjust based on who is playing
-- taken from original survival/Jotto
--------------------------------------------------------------------------
function ScenarioUtils.CreateResources()

	local Markers = ScenarioUtils.GetMarkers();

	for i, tblData in pairs(Markers) do -- loop marker list

		local SpawnThisResource = false; -- default to no

		if (tblData.resource and not tblData.SpawnWithArmy) then -- if this is a regular resource
			SpawnThisResource = true;
		elseif (tblData.resource and tblData.SpawnWithArmy) then -- if this is an army-specific resource

			if (tblData.SpawnWithArmy == "ARMY_0") then
				SpawnThisResource = true;
			else
				for x, army in ListArmies() do -- loop through army list

					if (tblData.SpawnWithArmy == army) then -- if this army is present
						SpawnThisResource = true; -- spawn this resource
						break;
					end
				end
			end
		end

		if (SpawnThisResource) then -- if we can spawn the resource do it

			local bp, albedo, sx, sz, lod;

			if (tblData.type == "Mass") then
				albedo = "/env/common/splats/mass_marker.dds";
				bp = "/env/common/props/massDeposit01_prop.bp";
				sx = 2;
				sz = 2;
				lod = 100;
			else
				albedo = "/env/common/splats/hydrocarbon_marker.dds";
				bp = "/env/common/props/hydrocarbonDeposit01_prop.bp";
				sx = 6;
				sz = 6;
				lod = 200;
			end

			-- create the resource
			CreateResourceDeposit(tblData.type,	tblData.position[1], tblData.position[2], tblData.position[3], tblData.size);

			-- create the resource graphic on the map
			CreatePropHPR(bp, tblData.position[1], tblData.position[2], tblData.position[3], Random(0,360), 0, 0);

			-- create the resource icon on the map
			CreateSplat(
				tblData.position,           # Position
				0,                          # Heading (rotation)
				albedo,                     # Texture name for albedo
				sx, sz,                     # SizeX/Z
				lod,                        # LOD
				0,                          # Duration (0 == does not expire)
				-1,                         # army (-1 == not owned by any single army)
				0							# ???
			);
		end
	end
end



-- called at start of game
--------------------------------------------------------------------------
function OnStart(self)

	LOG("----- Survival MOD: Initializing game start sequence...");

	-- start the survival tick
	ForkThread(Survival_Tick);

end



-- initializes the game settings
--------------------------------------------------------------------------
Survival_InitGame = function()

	LOG("----- Survival MOD: Configuring match settings...");

	-- check game configuration
	
		-- build time
		if (ScenarioInfo.Options.opt_Survival_BuildTime == nil) then
			ScenarioInfo.Options.opt_Survival_BuildTime = 150;
		end

			Survival_NextSpawnTime = ScenarioInfo.Options.opt_Survival_BuildTime; -- set first wave time to build time
			Survival_MinWarnTime = Survival_NextSpawnTime - 60; -- set time for minute warning

		-- opt_Survival_EnemiesPerMinute
		if (ScenarioInfo.Options.opt_Survival_EnemiesPerMinute == nil) then
			ScenarioInfo.Options.opt_Survival_EnemiesPerMinute = 24;
		end

		-- opt_Survival_WaveFrequency
		if (ScenarioInfo.Options.opt_Survival_WaveFrequency == nil) then
			ScenarioInfo.Options.opt_Survival_WaveFrequency = 20;
		end

		-- opt_Survival_Difficulty
--		if (ScenarioInfo.Options.opt_Survival_Difficulty == nil) then
--			ScenarioInfo.Options.opt_Survival_Difficulty = 1.00;
--		end

	ScenarioInfo.Options.Victory = 'sandbox'; -- force sandbox in order to implement our own rules

	--Utilities.UserConRequest("ui_ForceLifbarsOnEnemy"); -- force drawing of enemy life bars

	local Armies = ListArmies();
	Survival_PlayerCount = table.getn(Armies) - 2; -- save player count, subtracting the 2 AI "players"

	-- loop through armies
	for i, Army in ListArmies() do
		-- check if it's a human army
		if (Army == "ARMY_1" or Army == "ARMY_2" or Army == "ARMY_3" or Army == "ARMY_4" or Army == "ARMY_5" or Army == "ARMY_6" or Army == "ARMY_7" or Army == "ARMY_8") then 
			
			ScenarioFramework.AddRestriction(Army, categories.WALL); -- don't allow them to build walls
			ScenarioFramework.AddRestriction(Army, categories.AIR); -- don't allow them to build air stuff

			-- loop through other armies to ally with other human armies
			for x, ArmyX in ListArmies() do
				-- if human army
				if (ArmyX == "ARMY_1" or ArmyX == "ARMY_2" or ArmyX == "ARMY_3" or ArmyX == "ARMY_4" or ArmyX == "ARMY_5" or ArmyX == "ARMY_6" or ArmyX == "ARMY_7" or ArmyX == "ARMY_8") then 
					SetAlliance(Army, ArmyX, 'Ally'); 
				end
			end			

			SetAlliance(Army, "ARMY_SURVIVAL_ALLY", 'Ally'); -- friendly AI team
			SetAlliance(Army, "ARMY_SURVIVAL_ENEMY", 'Enemy');  -- enemy AI team

			SetAlliedVictory(Army, true); -- can win together of course :)
		end
	end

	SetAlliance("ARMY_SURVIVAL_ALLY", "ARMY_SURVIVAL_ENEMY", 'Enemy'); -- the friendly and enemy AI teams should be enemies

	SetIgnoreArmyUnitCap('ARMY_SURVIVAL_ENEMY', true); -- remove unit cap from enemy AI team

	Survival_InitMarkers(); -- find and reference all the map markers related to survival
	Survival_SpawnDef();

	Survival_CalcWaveCounts(); -- calculate how many units per wave

end



-- spawns a specified unit
--------------------------------------------------------------------------
Survival_InitMarkers = function()

	LOG("----- Survival MOD: Initializing marker lists...");

	local MarkerRef = nil;
	local Break = 0;
	local i = 1;

	while (Break < 3) do

		Break = 0; -- reset break counter

		-- center
		MarkerRef = GetMarker("SURVIVAL_CENTER_" .. i);

		if (MarkerRef != nil) then
			Survival_MarkerCounts[1] = Survival_MarkerCounts[1] + 1;
		else
			Break = Break + 1;
		end

		-- path
		MarkerRef = GetMarker("SURVIVAL_PATH_" .. i);

		if (MarkerRef != nil) then
			Survival_MarkerCounts[2] = Survival_MarkerCounts[2] + 1;
		else
			Break = Break + 1;
		end

		-- spawn
		MarkerRef = GetMarker("SURVIVAL_SPAWN_" .. i);

		if (MarkerRef != nil) then
			Survival_MarkerCounts[3] = Survival_MarkerCounts[3] + 1;
		else
			Break = Break + 1;
		end

		i = i + 1; -- increment counter

	end

	LOG("----- Survival MOD: Marker counts:     CENTER(" .. Survival_MarkerCounts[1] .. ")     PATHS(" .. Survival_MarkerCounts[2] .. ")     SPAWN(" .. Survival_MarkerCounts[3] .. ")");

end



-- spawns a defense object
--------------------------------------------------------------------------
Survival_SpawnDef = function()

	local POS = ScenarioUtils.MarkerToPosition("SURVIVAL_CENTER_1");
	Survival_DefUnit = CreateUnitHPR('XRB3301', "ARMY_SURVIVAL_ALLY", POS[1], POS[2], POS[3], 0,0,0);

	Survival_DefUnit:SetReclaimable(false);
	Survival_DefUnit:SetCapturable(false);
	Survival_DefUnit:SetProductionPerSecondEnergy((Survival_PlayerCount * 100) + 0);
	Survival_DefUnit:SetConsumptionPerSecondEnergy(0);

	Survival_DefUnit:SetMaxHealth(5500 - (Survival_PlayerCount * 500));
	Survival_DefUnit:SetHealth(nil, 5500 - (Survival_PlayerCount * 500));
	Survival_DefUnit:SetRegenRate(30 - (Survival_PlayerCount * 5));

	local Survival_DefUnitBP = Survival_DefUnit:GetBlueprint();
	Survival_DefUnitBP.Intel.MaxVisionRadius = 350;
	Survival_DefUnitBP.Intel.MinVisionRadius = 350;
	Survival_DefUnitBP.Intel.VisionRadius = 350;

	Survival_DefUnit:SetIntelRadius('Vision', 350);

       	local ShieldSpecs = {
            ImpactEffects = 'SeraphimShieldHit01',
            ImpactMesh = '/effects/entities/ShieldSection01/ShieldSection01_mesh',
            Mesh = '/effects/entities/SeraphimShield01/SeraphimShield01_mesh',
            MeshZ = '/effects/entities/Shield01/Shield01z_mesh',
            RegenAssistMult = 60,
            ShieldEnergyDrainRechargeTime = 60,
            ShieldMaxHealth = 55000 + (Survival_PlayerCount * 5000),
            ShieldRechargeTime = 60,
            ShieldRegenRate = 290 - (Survival_PlayerCount * 10),
            ShieldRegenStartTime = 1,
            ShieldSize = 90,
            ShieldVerticalOffset = -10,
        };

--	Survival_DefUnitBP.Defense.Shield = ShieldSpecs;

--	Survival_DefUnitBP.General.UnitName = 'Acen Accelerator';
--	Survival_DefUnitBP.Interface.HelpText = 'Special Operations Support';

	-- when the def object dies
	Survival_DefUnit.OldOnKilled = Survival_DefUnit.OnKilled;

	Survival_DefUnit.OnKilled = function(self, instigator, type, overkillRatio)

		BroadcastMSG("The defense object has been destroyed. You have lost!", 8);
		self.OldOnKilled(self, instigator, type, overkillRatio);

		Survival_GameState = 3;

		for i, army in ListArmies() do

			if (army == "ARMY_1" or army == "ARMY_2" or army == "ARMY_3" or army == "ARMY_4" or army == "ARMY_5" or army == "ARMY_6" or army == "ARMY_7" or army == "ARMY_8") then
				--killAllCommanders(army);
				GetArmyBrain(army):OnDefeat();
			end
		end
	end

	Survival_DefLastHP = Survival_DefUnit:GetHealth();

--	ScenarioFramework.CreateUnitDamagedTrigger(Survival_DefDamage, Survival_DefUnit);

--### Single Line unit damaged trigger creation
--# When <unit> is damaged it will call the <callbackFunction> provided
--# If <percent> provided, will check if damaged percent EXCEEDS number provided before callback
--# function repeats up to repeatNum ... or once if not declared
--function CreateUnitDamagedTrigger( callbackFunction, unit, amount, repeatNum )
--    TriggerFile.CreateUnitDamagedTrigger( callbackFunction, unit, amount, repeatNum )
--end

end



-- warns players about damage to defense object
--------------------------------------------------------------------------
--Survival_DefDamage = function()
--	BroadcastMSG("The Acen Accelerator is taking damage!");
--	LOG("----- Survival MOD: DefDamage()");
--	Survival_DefCheckHP = 0;
--	Survival_DefLastHP
--end



-- loops every TickInterval to progress main game logic
--------------------------------------------------------------------------
Survival_Tick = function(self)

	LOG("----- Survival MOD: Tick thread started with interval of (" .. Survival_TickInterval .. ")");

	while (Survival_GameState < 2) do

		Survival_CurrentTime = GetGameTimeSeconds();

		Survival_UpdateWaves(Survival_CurrentTime);

--		LOG("----- Survival MOD: -LOOP- GameState: " .. Survival_GameState .. "     NextSpawnTime: " .. SecondsToTime(Survival_NextSpawnTime) .. " (" .. Survival_NextSpawnTime .. ")     Clock:" .. SecondsToTime(Survival_CurrentTime) .. " (" .. Survival_CurrentTime .. ")");

		if (Survival_GameState == 0) then -- build stage

			if (Survival_CurrentTime >= Survival_NextSpawnTime) then -- if build period is over

				LOG("----- Survival MOD: Build state complete. Proceeding to combat state.");
				Sync.ObjectiveTimer = 0; -- clear objective timer
				Survival_GameState = 1; -- update game state to combat mode
				BroadcastMSG("Space Vikings are attacking!", 4);
				Survival_SpawnWave(Survival_NextSpawnTime);
				Survival_NextSpawnTime = Survival_NextSpawnTime + ScenarioInfo.Options.opt_Survival_WaveFrequency; -- update next wave spawn time by wave frequency

			else -- build period still active

				Sync.ObjectiveTimer = math.floor(Survival_NextSpawnTime - Survival_CurrentTime); -- update objective timer
				Survival_DefUnit:SetCustomName(SecondsToTime(Sync.ObjectiveTimer)); -- update defense object name

				if ((Survival_MinWarnTime > 0) and (Survival_CurrentTime >= Survival_MinWarnTime)) then -- display 2 minute warning if we're at 2 minutes and it's appropriate to do so
					LOG("----- Survival MOD: Sending 1 minute warning.");
					BroadcastMSG("1 minute warning!", 2);
					Survival_MinWarnTime = 0; -- reset 2 minute warning time so it wont be displayed again
				end

			end

		elseif (Survival_GameState == 1) then -- combat stage

			if (Survival_CurrentTime >= Survival_NextSpawnTime) then -- ready to spawn a wave
				Survival_SpawnWave(Survival_NextSpawnTime);
				Survival_NextSpawnTime = Survival_NextSpawnTime + ScenarioInfo.Options.opt_Survival_WaveFrequency; -- update next wave spawn time by wave frequency
			end

			Survival_DefUnit:SetCustomName('Wave Set ' ..  (Survival_WaveTables[1][1] - 1) .. "/" .. (table.getn(Survival_WaveTables[1]) - 1) ); -- .. ' (' .. SecondsToTime(Survival_CurrentTime - (Survival_WaveTables[1][Survival_WaveTables[1][1] + 1][1] * 60)).. ')');
			--SecondsToTime((Survival_WaveTables[1][(Survival_WaveTables[1][1])][1] * 60) - Survival_CurrentTime)
		end

		Survival_DefCheckHP = Survival_DefCheckHP - Survival_TickInterval;

		if (Survival_DefLastHP > Survival_DefUnit:GetHealth()) then
			if (Survival_DefCheckHP <= 0) then
				--BroadcastMSG("The Acen Accelerator is taking damage! (" .. Survival_DefUnit:GetHealth() / Survival_DefUnit:GetMaxHealth() .. "%)", 0.5);
				BroadcastMSG("The Acen Accelerator is taking damage! (" .. math.floor(Survival_DefUnit:GetHealthPercent() * 10) .. "%)", 0.5);

				Survival_DefCheckHP = 2;
			end
		end

		Survival_DefLastHP = Survival_DefUnit:GetHealth();

--		if (Survival_DefCheckHP <= 0) then
--			ScenarioFramework.CreateUnitDamagedTrigger(Survival_DefDamage, Survival_DefUnit);
--			Survival_DefCheckHP = 5.0;
--		else
--			Survival_DefCheckHP = Survival_DefCheckHP - Survival_TickInterval;
--		end

		WaitSeconds(Survival_TickInterval);
	end

	KillThread(self);

end



-- updates spawn waves
--------------------------------------------------------------------------
Survival_UpdateWaves = function(GameTime)

	local OldWaveID = 1;

	-- check the wave table times vs the wave spawn time to see which waves we spawn
	for x = 1, table.getn(Survival_WaveTables) do -- loop through each of the wavetable entries (ground/air/sea...)

		OldWaveID = 1;
		OldWaveID = Survival_WaveTables[x][1];

		for y = Survival_WaveTables[x][1], table.getn(Survival_WaveTables[x]) do -- loop through each wave table within the category

			if (GameTime >= (Survival_WaveTables[x][y][1] * 60)) then -- compare spawn time against the first entry spawn time for each wave table
				Survival_WaveTables[x][1] = y; -- update the wave id for this wave category
			else break; end

		end

		if (Survival_WaveTables[x][1] != OldWaveID) then -- if we have a new wave ID for this table
			LOG("----- Survival MOD: Updating wave table from C:" .. x .. " ID:" .. Survival_WaveTables[x][1] .. " ( Set:" .. (Survival_WaveTables[x][1] - 1) ..") at " .. SecondsToTime(GameTime));		
		end
	end
end



-- spawns a wave of units
--------------------------------------------------------------------------
Survival_SpawnWave = function(SpawnTime)

--	LOG("----- Survival MOD: Performing a wave spawn at " .. SecondsToTime(SpawnTime));

	local WaveTable = nil;
	local UnitTable = nil;

	local UnitID = nil;
	local OrderID = nil;
	local POS = nil;
	local RandID = nil;

	-- check the wave table times vs the wave spawn time to see which waves we spawn
	for x = 1, table.getn(Survival_WaveTables) do -- loop through each of the wavetable entries (ground/air/sea...)

--		LOG("----- Survival MOD: Category(" .. x .. ")     Wave Set (" .. Survival_WaveTables[x][1] - 1 .. ")   (ID: " .. Survival_WaveTables[x][1] .. ");

		-- for the amount of units we spawn in per wave
		for z = 1,Survival_UnitCountPerWave do
			
			WaveTable = Survival_WaveTables[x][Survival_WaveTables[x][1]]; -- grab the wave set table we're spawning from
			RandID = math.random(2, table.getn(WaveTable)); -- pick a random unit table from within this wave set
			UnitTable = WaveTable[RandID]; -- reference that unit table

			UnitID = Survival_GetUnitFromTable(UnitTable); -- pick a random unit id from this table
			OrderID = UnitTable[2]; -- get the order id from this unit table (always 2nd entry)

			POS = Survival_GetPOS(3, 25);

			Survival_SpawnUnit(UnitID, "ARMY_SURVIVAL_ENEMY", POS, OrderID);

		end

	end

end



-- spawns a specified unit
--------------------------------------------------------------------------
Survival_SpawnUnit = function(UnitID, ArmyID, POS, OrderID) -- blueprint, army, position, order

--	LOG("----- Survival MOD: SPAWNUNIT: Start function...");
	local PlatoonList = {};

	local NewUnit = CreateUnitHPR(UnitID, ArmyID, POS[1], POS[2], POS[3], 0,0,0);

	-- prevent wreckage from enemy units
--	local BP = NewUnit:GetBlueprint();
--	if (BP != nil) then
--		BP.Wreckage = nil;
--	end

	NewUnit:SetProductionPerSecondEnergy(325);

	table.insert(PlatoonList, NewUnit); -- add unit to a platoon
	Survival_PlatoonOrder(ArmyID, PlatoonList, OrderID); -- give the unit orders

end



-- returns a random unit from within a specified unit table
--------------------------------------------------------------------------
Survival_GetUnitFromTable = function(UnitTable)

	local RandID = math.random(3, table.getn(UnitTable));
	local UnitID = UnitTable[RandID];

	return UnitID;

end



-- returns a random spawn position
--------------------------------------------------------------------------
Survival_GetPOS = function(MarkerType, Randomization)

	local RandID = 1;
	local MarkerName = nil;

	RandID = math.random(1, Survival_MarkerCounts[MarkerType]);  -- get a random value from the selected marker count

 	-- 1 center / 2 waypoint / 3 spawn
 
	if (MarkerType == 1) then
		MarkerName = "SURVIVAL_CENTER_" .. RandID;
	elseif (MarkerType == 2) then
		MarkerName = "SURVIVAL_PATH_" .. RandID;
	elseif (MarkerType == 3) then
		MarkerName = "SURVIVAL_SPAWN_" .. RandID;
	else
		return nil;
	end

	local POS = Survival_RandomizePOS(ScenarioUtils.MarkerToPosition(MarkerName), Randomization);

	return POS;

end



-- test platoon order function
--------------------------------------------------------------------------
Survival_PlatoonOrder = function(ArmyID, UnitList, OrderID)	

--	LOG("----- Survival MOD: PLATOON: Start function...");

	if (UnitList == nil) then
		return;
	end

	local aiBrain = GetArmyBrain(ArmyID); --"ARMY_SURVIVAL_ENEMY");
	local aiPlatoon = aiBrain:MakePlatoon('','');
	aiBrain:AssignUnitsToPlatoon(aiPlatoon, UnitList, 'Attack', 'None'); -- platoon, unit list, "mission" and formation

 	-- 1 center / 2 waypoint / 3 spawn
 
 	if (OrderID == 4) then -- attack move / move

		-- attack move to random path
		POS = Survival_GetPOS(2, 25);
		aiPlatoon:AggressiveMoveToLocation(POS);

		-- move to random center
		POS = Survival_GetPOS(1, 25);
		aiPlatoon:MoveToLocation(POS, false);

 	elseif (OrderID == 3) then -- patrol paths

		-- move to random path
		POS = Survival_GetPOS(2, 25);
		aiPlatoon:MoveToLocation(POS, false);

		-- patrol to random path
		POS = Survival_GetPOS(2, 25);
		aiPlatoon:Patrol(POS);

	elseif (OrderID == 2) then -- attack move

		-- attack move to random path
		POS = Survival_GetPOS(2, 25);
		aiPlatoon:AggressiveMoveToLocation(POS);

		-- attack move to random center
		POS = Survival_GetPOS(1, 25);
		aiPlatoon:AggressiveMoveToLocation(POS);

	else -- default/order 1 is move

		-- move to random path
		POS = Survival_GetPOS(2, 25);
		aiPlatoon:MoveToLocation(POS, false);

		-- move to random center
		POS = Survival_GetPOS(1, 25);
		aiPlatoon:MoveToLocation(POS, false);
	end

end



-- calculates how many units to spawn per wave
--------------------------------------------------------------------------
function Survival_CalcWaveCounts()

	local WaveMultiplier = ScenarioInfo.Options.opt_Survival_WaveFrequency / 60;
	Survival_UnitCountPerMinute = ScenarioInfo.Options.opt_Survival_EnemiesPerMinute * Survival_PlayerCount;
	Survival_UnitCountPerWave = Survival_UnitCountPerMinute * WaveMultiplier;
	LOG("----- Survival MOD: CalcWaveCounts = ((" .. ScenarioInfo.Options.opt_Survival_EnemiesPerMinute .. " EPM * " .. Survival_PlayerCount .. " Players = " .. Survival_UnitCountPerMinute .. ")) * ((" .. ScenarioInfo.Options.opt_Survival_WaveFrequency .. " Second Waves / 60 = " .. WaveMultiplier .. ")) = " .. Survival_UnitCountPerWave .. " Units Per Wave     (( with Waves Per Minute of " .. (60 / ScenarioInfo.Options.opt_Survival_WaveFrequency) .. " = " .. (Survival_UnitCountPerWave * (60 / ScenarioInfo.Options.opt_Survival_WaveFrequency)) .. " of " .. Survival_UnitCountPerMinute .. " Units Per Minute.");
--	LOG("----- Survival MOD: CalcWaveCounts() accounts for " .. Survival_UnitCountPerWave .. " of " .. Survival_UnitCountPerMinute .. " units " .. (60 / ScenarioInfo.Options.opt_Survival_WaveFrequency) .. " times per minute.");

end



-- misc functions
--------------------------------------------------------------------------


-- returns hh:mm:ss from second count
-- taken from original survival script
SecondsToTime = function(Seconds)
	return string.format("%02d:%02d", math.floor(Seconds / 60), math.mod(Seconds, 60));
end

-- broadcast a text message to players
-- modified version of original survival script function
BroadcastMSG = function(MSG, Fade, TextColor)
	PrintText(MSG, 20, TextColor, Fade, 'center') ;	
end

-- gets map marker reference by name
-- taken from forum post by Saya
function GetMarker(MarkerName)
	return Scenario.MasterChain._MASTERCHAIN_.Markers[MarkerName]
end

-- returns a random spawn position
Survival_RandomizePOS = function(POS, x)

	local NewPOS = {0, 0, 0};

	NewPOS[1] = POS[1] + ((math.random() * (x * 2)) - x);
	NewPOS[3] = POS[3] + ((math.random() * (x * 2)) - x);

	return NewPOS;

end


--function OverrideDoDamage(self, instigator, amount, vector, damageType)
--    local preAdjHealth = self:GetHealth()
--    self:AdjustHealth(instigator, -amount)
--    local health = self:GetHealth()
--    if (( health <= 0 ) or ( amount > preAdjHealth )) and not self.KilledFlag then
--        self.KilledFlag = true
--        if( damageType == 'Reclaimed' ) then
--            self:Destroy()
--        else
--            local excessDamageRatio = 0.0
--            # Calculate the excess damage amount
--            local excess = preAdjHealth - amount
--            local maxHealth = self:GetMaxHealth()
--            if(excess < 0 and maxHealth > 0) then
--                excessDamageRatio = -excess / maxHealth
--            end
--            IssueClearCommands({self})
--            ForkThread( UnlockAndKillUnitThread, self, instigator, damageType, excessDamageRatio )
--        end
--    end
--end